#-*-coding:utf-8-*-

import os
import errno


def ensure_dir(path):
    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise


def ensure_file(path):
    """
    ensures that the file exists, also create the missing directories
    """
    basedir = os.path.dirname(path)
    ensure_dir(basedir)
    open(path, 'a').close()



#-*-coding:utf-8-*-

import os
from ConfigParser import SafeConfigParser
from pip.locations import legacy_config_file, config_basename
from pip.utils import appdirs
from pmm.utils import ensure_file


def get_pip_config_file_path():
    if os.path.exists(legacy_config_file):
        return legacy_config_file
    return os.path.join(appdirs('pip'), config_basename)


class PipConfig(object):

    def __init__(self):
        self.path = get_pip_config_file_path()
        self.config_parser = SafeConfigParser()
        if os.path.exists(self.path):
            self.config_parser.read(self.path)

    def get(self, section, name):
        if not self.config_parser.has_section(section):
            return
        return self.config_parser.get(section, name)

    def set(self, section, name, value):
        if not self.config_parser.has_section(section):
            self.config_parser.add_section(section)

        self.config_parser.set(section, name, value)

        ensure_file(self.path)
        with open(self.path, 'w') as fp:
            self.config_parser.write(fp)


pip_config = PipConfig()


def get_index_url():
    return pip_config.get('global', 'index-url')


def set_index_url(index_url):
    return pip_config.set('global', 'index-url', index_url)



#-*-coding:utf-8-*-

import os
import sys
from setuptools import setup

def fread(fname):
    filepath = os.path.join(os.path.dirname(__file__), fname)
    with open(filepath, 'r') as fp:
        return fp.read()


required = [
    'pip',
    'crayons',
    'blindspin',
    'requests',
    'requests-cache',
    'pick',
]

if sys.version_info < (2, 7):
    required.append('ordereddict')

setup(
    name='pmm',
    version='0.2.0',
    description='PyPi Mirror Manager',
    long_description=fread('README.md'),
    keywords='pypi,mirror',
    url='https://github.com/wong2/pmm',
    author='wong2',
    author_email='wonderfuly@gmail.com',
    license='MIT',
    install_requires=required,
    packages=['pmm'],
    entry_points={
        'console_scripts': ['pmm=pmm.cli:main'],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ]
)



#-*-coding:utf-8-*-

from __future__ import print_function

import blindspin
import crayons
from pick import pick

from pmm import config
from pmm.data import get_mirrors_data


def format_index_url(mirror):
    return 'https://{mirror}/simple'.format(mirror=mirror)


def main():
    with blindspin.spinner():
        mirrors = get_mirrors_data()

    current_index_url = config.get_index_url()

    current_index, choices = 0, []
    for i, mirror in enumerate(mirrors):
        if format_index_url(mirror) == current_index_url:
            current_index = i
        choices.append(mirror)

    choosed_mirror, _ = pick(choices, indicator='=>', default_index=current_index)

    choosed_index_url = format_index_url(choosed_mirror)
    config.set_index_url(choosed_index_url)

    print(u'✨  using mirror {} now'.format(crayons.cyan(choosed_index_url)))


if __name__ == '__main__':
    main()


#-*-coding:utf-8-*-

import requests
import requests_cache
from collections import OrderedDict

# cache api result for 10min
requests_cache.install_cache(expire_after=600)

STATUS_LABELS = OrderedDict([
    ('green', 'fresh'),
    ('yellow', 'aging'),
    ('red', 'old'),
    ('unavailable', 'n/a'),
])

def fetch_mirrors_data():
    api_url = 'https://www.pypi-mirrors.org/data.json'
    r = requests.get(api_url, verify=False)
    return r.json()


def get_mirrors_data():
    mirrors = []
    mirrors_data = fetch_mirrors_data()
    for mirror, data in mirrors_data.iteritems():
        mirrors.append((mirror, data['status'].lower()))
    mirrors.sort(key=lambda (mirror, status): list(STATUS_LABELS).index(status))
    return [mirror for mirror, status in mirrors]
